@@ -1,7 +1,10 @@
 """A device from the Smart Device Management API."""
 
+from __future__ import annotations
+
+import datetime
 import logging
-from typing import Awaitable, Callable
+from typing import Any, Awaitable, Callable, Dict, List, Mapping, Optional, cast
 
 # Import traits for registration
 from . import camera_traits  # noqa: F401
@@ -11,6 +14,7 @@ from . import thermostat_traits  # noqa: F401
 from .auth import AbstractAuth
 from .event import EventMessage, EventProcessingError, EventTrait
 from .traits import BuildTraits, Command
+from .typing import cast_assert
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -22,10 +26,10 @@ PARENT = "parent"
 DISPLAYNAME = "displayName"
 
 
-def _MakeEventTraitMap(traits: dict):
+def _MakeEventTraitMap(traits: Mapping[str, Any]) -> Dict[str, Any]:
     if camera_traits.CameraEventImageTrait.NAME not in traits:
         return {}
-    event_trait_map = {}
+    event_trait_map: Dict[str, Any] = {}
     for (trait_name, trait) in traits.items():
         if not hasattr(trait, "EVENT_NAME"):
             continue
@@ -36,23 +40,25 @@ def _MakeEventTraitMap(traits: dict):
 class Device:
     """Class that represents a device object in the Google Nest SDM API."""
 
-    def __init__(self, raw_data: dict, traits: dict):
+    def __init__(self, raw_data: Mapping[str, Any], traits: Dict[str, Any]):
         """Initialize a device."""
         self._raw_data = raw_data
         self._traits = traits
-        self._trait_event_ts = {}
+        self._trait_event_ts: Dict[str, datetime.datetime] = {}
         self._relations = {}
         for relation in self._raw_data.get(DEVICE_PARENT_RELATIONS, []):
             if PARENT not in relation or DISPLAYNAME not in relation:
                 continue
             self._relations[relation[PARENT]] = relation[DISPLAYNAME]
-        self._callbacks = []
+        self._callbacks: List[Callable[[EventMessage], Awaitable[None]]] = []
         self._event_trait_map = _MakeEventTraitMap(self._traits)
 
     @staticmethod
-    def MakeDevice(raw_data: dict, auth: AbstractAuth):
+    def MakeDevice(raw_data: Mapping[str, Any], auth: AbstractAuth) -> Device:
         """Create a device with the appropriate traits."""
         device_id = raw_data.get(DEVICE_NAME)
+        if not device_id:
+            raise ValueError(f"raw_data missing field '{DEVICE_NAME}'")
         cmd = Command(device_id, auth)
         traits = raw_data.get(DEVICE_TRAITS, {})
         traits_dict = BuildTraits(traits, cmd, raw_data.get(DEVICE_TYPE))
@@ -61,45 +67,40 @@ class Device:
     @property
     def name(self) -> str:
         """Resource name of the device such as 'enterprises/XYZ/devices/123'."""
-        return self._raw_data[DEVICE_NAME]
+        return cast_assert(str, self._raw_data[DEVICE_NAME])
 
     @property
-    def type(self) -> str:
+    def type(self) -> Optional[str]:
         """Type of device for display purposes.
 
         The device type should not be used to deduce or infer functionality of
         the actual device it is assigned to. Instead, use the returned traits for
         the device.
         """
-        return self._raw_data[DEVICE_TYPE]
+        return cast_assert(str, self._raw_data[DEVICE_TYPE])
 
     @property
-    def traits(self) -> dict:
+    def traits(self) -> Dict[str, Any]:
         """Return a trait mixin or None."""
         return self._traits
 
-    def _traits_data(self, trait) -> dict:
+    def _traits_data(self, trait: str) -> Dict[str, Any]:
         """Return the raw dictionary for the specified trait."""
         traits_dict = self._raw_data.get(DEVICE_TRAITS, {})
-        return traits_dict.get(trait, {})
+        return cast(Dict[str, Any], traits_dict.get(trait, {}))
 
     @property
     def parent_relations(self) -> dict:
         """Room or structure for the device."""
         return self._relations
 
-    @property
-    def raw_data(self) -> str:
-        """Return the raw data string."""
-        return self._raw_data
-
     def add_update_listener(self, target: Callable[[], None]) -> Callable[[], None]:
         """Register a simple event listener notified on updates.
 
         The return value is a callable that will unregister the callback.
         """
 
-        async def handle_event(event_message: EventMessage):
+        async def handle_event(event_message: EventMessage) -> None:
             target()
 
         return self.add_event_callback(handle_event)
@@ -113,7 +114,7 @@ class Device:
         """
         self._callbacks.append(target)
 
-        def remove_callback():
+        def remove_callback() -> None:
             """Remove the event_callback."""
             self._callbacks.remove(target)
 
@@ -130,13 +131,16 @@ class Device:
             raise EventProcessingError(
                 f"Mismatch {self.name} != {event_message.resource_update_name}"
             )
-        traits = event_message.resource_update_traits
-        if traits:
-            _LOGGER.debug("Trait update %s", traits.keys())
-        events = event_message.resource_update_events
-        if events:
-            _LOGGER.debug("Event Update %s", events.keys())
+        self._async_handle_traits(event_message)
+        self._async_handle_events(event_message)
+        for callback in self._callbacks:
+            await callback(event_message)
 
+    def _async_handle_traits(self, event_message: EventMessage) -> None:
+        traits = event_message.resource_update_traits
+        if not traits:
+            return
+        _LOGGER.debug("Trait update %s", traits.keys())
         for (trait_name, trait) in traits.items():
             # Discard updates older than prior events
             # Note: There is still a race where traits read from the API on
@@ -149,15 +153,17 @@ class Device:
             self._traits[trait_name] = trait
             self._trait_event_ts[trait_name] = event_message.timestamp
 
+    def _async_handle_events(self, event_message: EventMessage) -> None:
+        events = event_message.resource_update_events
+        if not events:
+            return
+        _LOGGER.debug("Event Update %s", events.keys())
         for (event_name, event) in events.items():
             if event_name not in self._event_trait_map:
                 continue
             self._event_trait_map[event_name].handle_event(event)
 
-        for callback in self._callbacks:
-            await callback(event_message)
-
-    def active_events(self, event_types: list) -> {}:
+    def active_events(self, event_types: list) -> dict:
         """Return any active events for the specified trait names."""
         active_events = {}
         for event_type in event_types:
@@ -168,9 +174,9 @@ class Device:
         return active_events
 
     @property
-    def active_event_trait(self) -> EventTrait:
+    def active_event_trait(self) -> Optional[EventTrait]:
         """Return trait with the most recently received active event."""
-        trait_to_return = None
+        trait_to_return: EventTrait | None = None
         for trait in self._event_trait_map.values():
             if not trait.active_event:
                 continue
@@ -178,6 +184,13 @@ class Device:
                 trait_to_return = trait
             else:
                 event = trait.last_event
+                if not event or not trait_to_return.last_event:
+                    continue
                 if event.expires_at > trait_to_return.last_event.expires_at:
                     trait_to_return = trait
         return trait_to_return
+
+    @property
+    def raw_data(self) -> Dict[str, Any]:
+        """Return raw data for the device."""
+        return dict(self._raw_data)
